@@ -139,6 +139,10 @@ class Resource:
             cache files that might not be updated with the latest github commit.
 
         """
+        if not fname:
+            log.warning(f"A file has value False, assuming this is intentional.")
+            return
+
         if fname.startswith('/'):
             log.warning(f"Using local file {fname} for a resource. "
                         f"Do not set this as a default or TravisCI tests will break")
@@ -226,10 +230,19 @@ class Resource:
 
         elif config.get('detector', 'XENONnT') == 'XENONnT':
             self.s1_pattern_map = make_map(files['s1_pattern_map'], fmt='pkl')
-            self.s1_lce_correction_map = make_map(files['s1_lce_correction_map'])
             self.s2_pattern_map = make_map(files['s2_pattern_map'], fmt='pkl')
             self.s2_correction_map = make_map(files['s2_correction_map'])
 
+            #if there is a (data driven!) map, load it. If not make it  from the pattern map
+            if files['s1_lce_correction_map']:
+                self.s1_lce_correction_map = make_map(files['s1_lce_correction_map'])
+
+            else:
+                lymap = deepcopy(self.s1_pattern_map)
+                lymap.data['map'] = np.sum(lymap.data['map'][:][:][:], axis=3, keepdims=True)
+                lymap.__init__(lymap.data)
+                self.s1_lce_correction_map = lymap
+
             # Garfield luminescence timing samples
             if config.get('s2_luminescence_model', False) == 'garfield':
                 s2_luminescence_map = straxen.get_resource(files['s2_luminescence'], fmt='npy_pickle')['arr_0']
@@ -239,9 +252,12 @@ class Resource:
                 liquid_level = min(liquid_level_available, key=lambda x: abs(x - liquid_level))
                 self.s2_luminescence = s2_luminescence_map[s2_luminescence_map['ll'] == liquid_level]
 
-            if config.get('field_distortion_on', False):
+            if config.get('field_distortion_model', "none") == "inverse_fdc":
                 self.fdc_3d = make_map(files['fdc_3d'], fmt='json.gz')
 
+            if config.get('field_distortion_model', "none") == "comsol":
+                self.fd_comsol = make_map(config['field_distortion_comsol_map'], fmt='json.gz', method='RectBivariateSpline')
+
             # Gas gap warping map
             if config.get('enable_gas_gap_warping', False):
                 gas_gap_map = straxen.get_resource(files['gas_gap_map'], fmt='pkl')
@@ -250,7 +266,7 @@ class Resource:
             # Field dependencies 
             # This config entry a dictionary of 5 items
             if any(config['enable_field_dependencies'].values()):
-                field_dependencies_map = make_map(files['field_dependencies_map'], fmt='json.gz')
+                field_dependencies_map = make_map(files['field_dependencies_map'], fmt='json.gz', method='RectBivariateSpline')
 
                 def rz_map(z, xy, **kwargs):
                     r = np.sqrt(xy[:, 0]**2 + xy[:, 1]**2)
@@ -272,7 +288,6 @@ class Resource:
 
         elif config.get('detector', 'XENONnT') == 'XENONnT_neutron_veto':
             # Neutron veto PMT QE as function of wavelength
-            if config.get('neutron_veto', False):
                 self.nv_pmt_qe = straxen.get_resource(files['nv_pmt_qe'], fmt='json')
 
         # SPE area distributions
